@@ -13,6 +13,9 @@ import { LightningStrike } from '/examples/jsm/geometries/LightningStrike.js';
 import { OrbitControls } from '/examples/jsm/controls/OrbitControls.js';
 import { LineMaterial } from './jsm/lines/LineMaterial.js';
 import { LineGeometry } from './jsm/lines/LineGeometry.js';
+import { FBXLoader } from './jsm/loaders/FBXLoader.js';
+import { Font, FontLoader } from './jsm/loaders/FontLoader.js';
+import { TTFLoader } from './jsm/loaders/TTFLoader.js';
 import { Line2 } from './jsm/lines/Line2.js';
 window.glb = Object()
 window.glb.clock = new THREE.Clock();
@@ -22,6 +25,7 @@ window.glb.BarFolder = null;
 window.glb.camera = null;
 window.glb.camera2 = null;
 window.glb.stats = null;
+window.glb.font = null;
 
 window.glb.import_Stats = Stats;
 window.glb.import_GUI = GUI;
@@ -30,23 +34,29 @@ window.glb.import_LightningStrike = LightningStrike;
 window.glb.import_LineMaterial = LineMaterial;
 window.glb.import_LineGeometry = LineGeometry;
 window.glb.import_Line2 = Line2;
+window.glb.import_Font = Font;
+window.glb.import_FontLoader = FontLoader;
+window.glb.import_TTFLoader = TTFLoader;
+window.glb.import_FBXLoader = FBXLoader;
 window.glb.renderer = null;
 window.glb.controls=null;
 window.glb.controls2=null;
 // var window.glb.scene;
-// 历史轨迹
 window.glb.core_L = [];
 window.glb.parsed_core_L = []
 window.glb.core_Obj = [];
 window.glb.line_Obj = [];
+window.glb.text_Obj = [];
 window.glb.flash_Obj = [];
 window.glb.base_geometry = {};
+window.glb.base_material = {};
 // global vars
 window.glb.play_fps = 5;
 window.glb.play_pointer = 0; // 位置读取指针
 window.glb.solid_pointer = 0; // 过渡动画前置位置的指针
 window.glb.sp_future = 0;   // 用于辅助确定 solid_pointer
 window.glb.dt_threshold = 1 / window.glb.play_fps;
+var client_uuid = generateUUID();
 
 var dt_since = 0;
 var buf_str = '';
@@ -57,16 +67,19 @@ var ppt_mode = 0;
 var request=null;
 var req_interval=2.0;
 window.glb.panelSettings = {
-        'play fps': window.glb.play_fps,
-        'play pointer':0,
-        'data req interval': req_interval,
-        'reset to read new': null,
-        'pause': null,
-        'next frame': null,
-        'previous frame': null,
-        'ppt step': null,
-        'loop to start': false,
-        'use orthcam': false
+    'play fps': window.glb.play_fps,
+    'play pointer':0,
+    'data req interval': req_interval,
+    'reset to read new': null,
+    'pause': null,
+    'next frame': null,
+    'previous frame': null,
+    'freeze': null,
+    'ppt step': null,
+    'loop to start': false,
+    'smooth control': true,
+    'show camera orbit': false,
+    'use orthcam': false
 };
 
 
@@ -112,7 +125,7 @@ var coreReadFunc = function (auto_next=true) {
 
         transfer_ongoing = false;
     };
-    request.send();
+    request.send(client_uuid);
     // console.log('send req')
     transfer_ongoing = true;
     if(auto_next && !DEBUG){setTimeout(coreReadFunc, req_interval*1000);}
@@ -133,12 +146,12 @@ function change_fps(fps) {
     window.glb.dt_threshold = 1 / window.glb.play_fps;
 }
 
-function pause_play(){
+function pause_play(test_freeze=false){
     if(window.glb.panelSettings['play fps']>0){
         window.glb.prev_fps = window.glb.panelSettings['play fps'];
     }
     window.glb.play_pointer = window.glb.solid_pointer;
-    force_move_all(window.glb.solid_pointer)
+    force_move_all(window.glb.solid_pointer, test_freeze)
     window.glb.panelSettings['play fps'] = 0; 
     change_fps(0);
 }
@@ -156,7 +169,13 @@ window.glb.panelSettings['pause'] = function (){
         resume_play()
     }
 }
-
+window.glb.panelSettings['freeze'] = function (){
+    if (window.glb.panelSettings['play fps']>0){
+        pause_play(true)
+    }else{
+        resume_play()
+    }
+}
 window.glb.panelSettings['next frame'] = function (){
     // pause play
     if(window.glb.play_fps!=0){
@@ -262,7 +281,8 @@ function check_flash_life_cyc(delta_time){
 
 
 
-
+// force_move_all(pp, test_freeze=false) --> change_position_rotation_size(object, 1, true, true)
+// move_to_future(percent, override=false)  --> change_position_rotation_size(object, percent, override, false)
 function change_position_rotation_size(object, percent, override, reset_track=false){
     object.position.x = object.prev_pos.x * (1 - percent) + object.next_pos.x * percent
     object.position.y = object.prev_pos.y * (1 - percent) + object.next_pos.y * percent
@@ -277,7 +297,11 @@ function change_position_rotation_size(object, percent, override, reset_track=fa
 
     let size = object.prev_size * (1 - percent)  + object.next_size * percent
     let opacity = object.prev_opacity * (1 - percent)  + object.next_opacity * percent
-    if (object.material.opacity!=opacity){ object.material.opacity=opacity }
+    if (object.material.opacity!=opacity){ 
+        object.material.opacity = opacity;
+        object.material.transparent = (opacity==1)?false:true;
+        object.renderOrder = (opacity==0)?256:object.renderOrder;
+    }
     changeCoreObjSize(object, size)
 
     if(override){
@@ -371,16 +395,31 @@ function set_next_play_frame() {
         window.glb.play_pointer = window.glb.panelSettings['loop to start']?0:window.glb.core_L.length-1;
     }
 }
-function force_move_all(pp){ // 手动调整进度条时触发
+
+function force_move_all(pp, test_freeze=false){ // 手动调整进度条时触发
     parse_time_step(pp)
     window.glb.solid_pointer = pp; window.glb.panelSettings['play pointer'] = window.glb.solid_pointer; window.glb.sp_future = pp;
     console.log('set_next_play_frame sp:'+window.glb.solid_pointer+' spf:'+window.glb.sp_future)
     for (let i = 0; i < window.glb.core_Obj.length; i++) {
         let object = window.glb.core_Obj[i]
-        object.prev_pos.x = object.next_pos.x; object.prev_pos.y = object.next_pos.y; object.prev_pos.z = object.next_pos.z
-        object.prev_ro.x = object.next_ro.x; object.prev_ro.y = object.next_ro.y; object.prev_ro.z = object.next_ro.z
-        object.prev_opacity = object.next_opacity; object.prev_size = object.next_size;
-        change_position_rotation_size(object, 1, true, true)
+        if(!test_freeze){
+            object.prev_pos.x = object.next_pos.x; object.prev_pos.y = object.next_pos.y; object.prev_pos.z = object.next_pos.z
+            object.prev_ro.x = object.next_ro.x; object.prev_ro.y = object.next_ro.y; object.prev_ro.z = object.next_ro.z
+            object.prev_opacity = object.next_opacity; object.prev_size = object.next_size;
+        }else{
+            object.prev_pos = object.position.clone()
+            object.next_pos = object.position.clone()
+            object.prev_ro.x = object.rotation.x; object.prev_ro.y = object.rotation.y; object.prev_ro.z = object.rotation.z
+            object.next_ro.x = object.rotation.x; object.next_ro.y = object.rotation.y; object.next_ro.z = object.rotation.z
+            // object.prev_ro = object.rotation.clone()
+            // object.next_ro = object.rotation.clone()
+            object.prev_opacity = object.material.opacity
+            object.next_opacity = object.material.opacity
+            object.prev_size = object.currentSize
+            object.next_size = object.currentSize
+        }
+        let reset_track = !test_freeze
+        change_position_rotation_size(object, 1, true, reset_track)
     }	
 }
 window.glb.force_move_all = force_move_all
@@ -400,6 +439,7 @@ function animate() {
     requestAnimationFrame(animate);
     render();
     window.glb.stats.update();
+    window.glb.controls.update()
 }
 
 function start_transition(){
@@ -432,12 +472,68 @@ function render() {
     }
     check_flash_life_cyc(delta)
 
+    sprite_like_txt_update()
+
     if(window.glb.panelSettings['use orthcam']) {window.glb.renderer.render(window.glb.scene, window.glb.camera2);}
-    else{window.glb.renderer.render(window.glb.scene, window.glb.camera);}
+    else{
+        show_cam_orbit();
+        window.glb.renderer.render(window.glb.scene, window.glb.camera);
+    }
+
 }
 
+function sprite_like_txt_update(){
+    var lookAtVector = new THREE.Vector3(0, 0, -1);
+    lookAtVector.applyQuaternion(window.glb.camera.quaternion);
+    for (let i = 0; i < window.glb.text_Obj.length; i++) {
+        let text_object = window.glb.text_Obj[i]
+        let target = new THREE.Vector3(); 
+        text_object.getWorldPosition(target)
+        text_object.lookAt(target.add(lookAtVector.clone().negate()));
+        let object = text_object.parent;
+        let new_rel_pos;
+        if (!object.label_offset){
+            new_rel_pos = new THREE.Vector3(object.generalSize, object.generalSize, -object.generalSize);
+        }else{
+            new_rel_pos = new THREE.Vector3(object.label_offset[0], object.label_offset[2], -object.label_offset[1]);
+        }
+        let q = object.quaternion.clone().invert()
+        new_rel_pos.applyQuaternion(q)
+        text_object.position.set(new_rel_pos.x,new_rel_pos.y,new_rel_pos.z)
+    }
+}
 
-
+var cam_orbit_deleted = true;
+function show_cam_orbit(){
+    if(window.glb.panelSettings['show camera orbit']) {
+        let target = window.glb.controls.target
+        let distance = window.glb.controls.getDistance()
+        apply_simple_line_update(find_lineobj_by_id('cam_x'), {
+            'x_arr': [target.x,target.x+distance*0.2],
+            'y_arr': [target.y,target.y],
+            'z_arr': [target.z,target.z],
+            'my_id': 'cam_x', 'color_str': 'Red',
+        });
+        apply_simple_line_update(find_lineobj_by_id('cam_y'),{
+            'x_arr': [target.x,target.x],
+            'y_arr': [target.y,target.y+distance*0.2],
+            'z_arr': [target.z,target.z],
+            'my_id': 'cam_y', 'color_str': 'Blue',
+        });
+        apply_simple_line_update(find_lineobj_by_id('cam_z'),{
+            'x_arr': [target.x,target.x],
+            'y_arr': [target.y,target.y],
+            'z_arr': [target.z-distance*0.2,target.z],
+            'my_id': 'cam_z', 'color_str': 'Green',
+        });
+        cam_orbit_deleted = false
+    }else if(!cam_orbit_deleted){
+        let cam_x = find_lineobj_by_id('cam_x'); if (cam_x){window.glb.scene.remove(cam_x.mesh);window.glb.line_Obj.remove(cam_x);}
+        let cam_y = find_lineobj_by_id('cam_y'); if (cam_y){window.glb.scene.remove(cam_y.mesh);window.glb.line_Obj.remove(cam_y);}
+        let cam_z = find_lineobj_by_id('cam_z'); if (cam_z){window.glb.scene.remove(cam_z.mesh);window.glb.line_Obj.remove(cam_z);}
+        cam_orbit_deleted=true;
+    }
+}
 
 
 init();
